@@ -247,7 +247,7 @@ class SmsEntryWindow(object):
 
 	MAX_CHAR = 160
 
-	def __init__(self, widgetTree, parent):
+	def __init__(self, widgetTree, parent, app):
 		self._clipboard = gtk.clipboard_get()
 		self._widgetTree = widgetTree
 		self._window = self._widgetTree.get_widget("smsWindow")
@@ -257,6 +257,11 @@ class SmsEntryWindow(object):
 		self._isFullScreen = False
 		self._parent = parent
 
+		errorBox = self._widgetTree.get_widget("smsErrorEventBox")
+		errorDescription = self._widgetTree.get_widget("smsErrorDescription")
+		errorClose = self._widgetTree.get_widget("smsErrorClose")
+		self._errorDisplay = gtk_toolbox.ErrorDisplay(errorBox, errorDescription, errorClose)
+
 		self._smsButton = self._widgetTree.get_widget("sendSmsButton")
 		self._smsButton.connect("clicked", self._on_send)
 		self._dialButton = self._widgetTree.get_widget("dialButton")
@@ -290,18 +295,25 @@ class SmsEntryWindow(object):
 		self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
 		self._smsEntrySize = None
 
+		self._app = app
+		self._window = hildonize.hildonize_window(self._app, self._window)
+
 		self._contacts = []
 
-	def add_contact(self, contactDetails, messages = (), defaultIndex = -1):
+	def add_contact(self, name, contactDetails, messages = (), defaultIndex = -1):
 		contactNumbers = list(self._to_contact_numbers(contactDetails))
 		assert contactNumbers
 		contactIndex = defaultIndex if defaultIndex != -1 else 0
 		contact = contactNumbers, contactIndex, messages
 		self._contacts.append(contact)
 
+		nameLabel = gtk.Label(name)
 		selector = gtk.Button(contactNumbers[0][1])
+		if len(contactNumbers) == 1:
+			selector.set_sensitive(False)
 		removeContact = gtk.Button(stock="gtk-delete")
 		row = gtk.HBox()
+		row.pack_start(nameLabel, True, True)
 		row.pack_start(selector, True, True)
 		row.pack_start(removeContact, False, False)
 		row.show_all()
@@ -325,8 +337,8 @@ class SmsEntryWindow(object):
 	def clear(self):
 		del self._contacts[:]
 
-		for contactNumberSelector in list(self._targetList.get_children()):
-			self._targetList.remove(contactNumberSelector)
+		for row in list(self._targetList.get_children()):
+			self._targetList.remove(row)
 		self._smsEntry.get_buffer().set_text("")
 		self._update_letter_count()
 		self._update_context()
@@ -340,8 +352,8 @@ class SmsEntryWindow(object):
 	def _remove_contact(self, contactIndex):
 		del self._contacts[contactIndex]
 
-		contactNumberSelector = list(self._targetList.get_children())[contactIndex]
-		self._targetList.remove(contactNumberSelector)
+		row = list(self._targetList.get_children())[contactIndex]
+		self._targetList.remove(row)
 		self._update_button_state()
 		self._update_context()
 
@@ -442,10 +454,10 @@ class SmsEntryWindow(object):
 			contactNumbers, numberIndex, messages = self._contacts[0]
 			self._phoneButton.set_label(contactNumbers[numberIndex][1])
 			row = list(self._targetList.get_children())[0]
-			phoneButton = list(row.get_children())[0]
+			phoneButton = list(row.get_children())[1]
 			phoneButton.set_label(contactNumbers[numberIndex][1])
-		except Exception, e:
-			_moduleLogger.exception("%s" % str(e))
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_choose_phone_n(self, button, row):
 		try:
@@ -455,10 +467,10 @@ class SmsEntryWindow(object):
 			self._request_number(index)
 
 			contactNumbers, numberIndex, messages = self._contacts[0]
-			phoneButton = list(row.get_children())[0]
+			phoneButton = list(row.get_children())[1]
 			phoneButton.set_label(contactNumbers[numberIndex][1])
-		except Exception, e:
-			_moduleLogger.exception("%s" % str(e))
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_remove_phone_n(self, button, row):
 		try:
@@ -470,43 +482,52 @@ class SmsEntryWindow(object):
 			self._targetList.remove(row)
 			self._update_context()
 			self._update_button_state()
-		except Exception, e:
-			_moduleLogger.exception("%s" % str(e))
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_entry_changed(self, *args):
-		self._update_letter_count()
+		try:
+			self._update_letter_count()
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_send(self, *args):
-		assert 0 < len(self._contacts), "%r" % self._contacts
-		phoneNumbers = [
-			make_ugly(contact[0][contact[1]][0])
-			for contact in self._contacts
-		]
+		try:
+			assert 0 < len(self._contacts), "%r" % self._contacts
+			phoneNumbers = [
+				make_ugly(contact[0][contact[1]][0])
+				for contact in self._contacts
+			]
 
-		entryBuffer = self._smsEntry.get_buffer()
-		enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
-		enteredMessage = enteredMessage.strip()
-		assert enteredMessage
-		self.send_sms(phoneNumbers, enteredMessage)
-		self._hide()
+			entryBuffer = self._smsEntry.get_buffer()
+			enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
+			enteredMessage = enteredMessage.strip()
+			assert enteredMessage
+			self.send_sms(phoneNumbers, enteredMessage)
+			self._hide()
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_dial(self, *args):
-		assert len(self._contacts) == 1, "%r" % self._contacts
-		contact = self._contacts[0]
-		contactNumber = contact[0][contact[1]][0]
-		phoneNumber = make_ugly(contactNumber)
-		self.dial(phoneNumber)
-		self._hide()
+		try:
+			assert len(self._contacts) == 1, "%r" % self._contacts
+			contact = self._contacts[0]
+			contactNumber = contact[0][contact[1]][0]
+			phoneNumber = make_ugly(contactNumber)
+			self.dial(phoneNumber)
+			self._hide()
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 	def _on_delete(self, *args):
-		self._window.emit_stop_by_name("delete-event")
-		self._hide()
+		try:
+			self._window.emit_stop_by_name("delete-event")
+			self._hide()
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 		return True
 
 	def _on_window_state_change(self, widget, event, *args):
-		"""
-		@note Hildon specific
-		"""
 		try:
 			if event.new_window_state & gtk.gdk.WINDOW_STATE_FULLSCREEN:
 				self._isFullScreen = True
@@ -542,8 +563,8 @@ class SmsEntryWindow(object):
 				event.get_state() & gtk.gdk.CONTROL_MASK
 			):
 				self._parent.destroy()
-		except Exception, e:
-			_moduleLogger.exception(str(e))
+		except TypeError, e:
+			self._errorDisplay.push_exception()
 
 
 class Dialpad(object):
@@ -636,6 +657,7 @@ class Dialpad(object):
 		try:
 			phoneNumber = self.get_number()
 			self.add_contact(
+				"(Dialpad)",
 				[("Dialer", phoneNumber)], ()
 			)
 			self.set_number("")
@@ -1191,10 +1213,12 @@ class CallHistoryView(object):
 			prettyNumber = self._historymodel.get_value(itr, self.NUMBER_IDX)
 			number = make_ugly(prettyNumber)
 			description = list(self._history_summary(prettyNumber))
+			contactName = self._historymodel.get_value(itr, self.FROM_IDX)
 			contactId = self._historymodel.get_value(itr, self.FROM_ID_IDX)
 			contactPhoneNumbers, defaultIndex = _get_contact_numbers(self._backend, contactId, number)
 
 			self.add_contact(
+				contactName,
 				contactPhoneNumbers,
 				messages = description,
 				defaultIndex = defaultIndex,
@@ -1436,9 +1460,11 @@ class MessagesView(object):
 			description = self._messagemodel.get_value(itr, self.MESSAGES_IDX)
 
 			contactId = self._messagemodel.get_value(itr, self.FROM_ID_IDX)
+			header = self._messagemodel.get_value(itr, self.HEADER_IDX)
 			contactPhoneNumbers, defaultIndex = _get_contact_numbers(self._backend, contactId, number)
 
 			self.add_contact(
+				header,
 				contactPhoneNumbers,
 				messages = description,
 				defaultIndex = defaultIndex,
@@ -1712,6 +1738,7 @@ class ContactsView(object):
 				return
 
 			self.add_contact(
+				contactName,
 				contactPhoneNumbers,
 				messages = (contactName, ),
 			)
